/*
 * grid.java
 *
 * Created on 26. srpen 2003, 22:42
 */
import java.awt.*;
import java.applet.*;

/** Class representing the grid where caterpillar is moving and where barriers are
 * placed.
 * @author Pavel
 */
public class Grid {
    private int width;
    private int height;
    
    private int oldWidth;
    private int oldHeight;
    private int oldLeft;
    private int oldTop;
    private boolean eraseNeeded = false;
    /** Width and height (cells are squares) of interior of a cell. */    
    public static int cellSize = 12;
    private Color gridColor = Color.black;
    
    /** Creates a new instance of grid
     * @param width Width of a grid (number of cells).
     * @param height Height of a grid (number of cells).
     */
    public Grid(int width, int height) {
        this.width = width;
        this.height = height;
    }
    
    /** Call this method to repaint the grid.
     * @param backgroundColor Background color of a grid. Is used to clear grid if it is reduced.
     * @param g Graphics object used to painting.
     * @param left x-coordinate of upper-left corner of a grid (in pixels).
     * @param top y-coordinate of upper-left corner of a grid (in pixels).
     */    
    public void Paint(Color backgroundColor, Graphics g, int left, int top)
    {
        if (eraseNeeded)
        {
            PaintWithColor(g, oldLeft, oldTop, backgroundColor, oldWidth, oldHeight);
            eraseNeeded = false;
        }
        PaintWithColor(g, left, top, gridColor, this.width, this.height);
    }
    
    private void PaintWithColor(Graphics g, int left, int top, Color c, int width, int height)
    {
        g.setColor(c);
        
        int right = left + GetWidthPixels() - 1;
        g.drawLine(left, top, right, top);
        int y = top + 1;
        for (int i = 0; i < height; i++)
        {
            g.drawLine(left, y, right, y);
            y += cellSize + 1;
        }
//        y += cellSize + 1;
        g.drawLine(left, y, right, y);
        y++;
        g.drawLine(left, y, right, y);
        
        int bottom = top + GetHeightPixels() - 1;
        g.drawLine(left, top, left, bottom);
        int x = left + 1;
        for (int i = 0; i < width; i++)
        {
            g.drawLine(x, top, x, bottom);
            x += cellSize + 1;
        }
//        x += cellSize + 1;
        g.drawLine(x, top, x, bottom);
        x++;
        g.drawLine(x, top, x, bottom);
        
        oldLeft = left;
        oldTop = top;
    }
    
    /** Gets the width of a grid in pixels.
     * @return The whole width of a grid in pixels.
     */    
    public int GetWidthPixels()
    {
        return width * (cellSize + 1) + 3;
    }
    
    /** Gets the height of a grid in pixels.
     * @return The whole height of a grid in pixels.
     */    
    public int GetHeightPixels()
    {
        return height * (cellSize + 1) + 3;
    }
    
    /** Sets the width of a grid.
     * @param width Width of a grid (number of cells).
     * @throws IllegalArgumentException Throws if width is less than 1 or greater than 50.
     */    
    public void SetWidth(int width) throws IllegalArgumentException
    {
        if ((width < 10) || (width > 50))
            throw new IllegalArgumentException("Value 10 - 50 required.");
        this.oldWidth = this.width;
        this.width = width;
        eraseNeeded = true;
    }

    /** Sets the height of a grid.
     * @param height Height of a grid (number of cells).
     * @throws IllegalArgumentException Throws if height is less than 1 or greater than 50.
     */    
    public void SetHeight(int height) throws IllegalArgumentException
    {
        if ((height < 10) || (height > 50))
            throw new IllegalArgumentException("Value 10 - 50 required.");
        this.oldHeight = this.height;
        this.height = height;
        eraseNeeded = true;
    }
    
    /** Gets the width of a grid (number of cells).
     * @return Wdith of a grid.
     */    
    public int GetWidth()
    {
        return width;
    }
    
    /** Gets the height of a grid (number of cells).
     * @return Height of a grid.
     */    
    public int GetHeight()
    {
        return height;
    }
    
    /** Counts the upper-left corner point of a interior of a specific cell.
     * @param cellX Column number. Zero-based.
     * @param cellY Row number. Zero-based.
     * @param positionX X-coordinate (in pixels) of upper-left corner of a grid.
     * @param positionY Y-coordinate (in pixels) of upper-left corner of a grid.
     * @return upper-left corner point of a interior of a specific cell
     */    
    public static Point countCellPosition(int cellX, int cellY, int positionX, int positionY)
    {
        return new Point(positionX + 2 + cellX * (cellSize + 1),
                        positionY + 2 + cellY * (cellSize + 1));
    }
    
}
